unit Assembler;

(***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************)

interface

uses
  Mars,
  SysUtils;

type
  EAssembler = class(Exception);

// type to string
function mnemonic(AOpCode: TOpCode): string; overload; // always 3 chars, captials
function mnemonic(AOpModifier: TOpModifier): string; overload; // always right-padded to 2 chars
function mnemonic(AAddressingMode: TAddressingMode): char; overload;

// string to type; might throw EAssembler
function OpCode(s: string): TOpCode;
function OpModifier(s: string): TOpModifier;
function AddrMode(ch: char): TAddressingMode;
function Value(s: string): TOperand;

implementation

const
  OpCodeLookup: array[TOpCode] of string =
    ('DAT','MOV','ADD','SUB','MUL','DIV','MOD','JMP','JMZ','JMN','DJN','SPL','SLT','SEQ','SNE','NOP','LDP','STP');
  OpModifierLookup: array[TOpModifier] of string =
    ('A ','B ','AB','BA','F ','X ','I ');
  AddressingModeLookup: array[TAddressingMode] of Char =
    ('#','$','@','<','>','*','{','}');

function mnemonic(AOpCode: TOpCode): string;
begin
  if (AOpCode >= Low(TOpCode)) and (AOpCode <= High(TOpCode)) then
    Result := OpCodeLookup[AOpCode]
  else
    Result := ConCat('!',IntToStr(integer(AOpCode)));
  end;

function mnemonic(AOpModifier: TOpModifier): string;
begin Result := OpModifierLookup[AOpModifier]; end;

function mnemonic(AAddressingMode: TAddressingMode): char;
begin Result := AddressingModeLookup[AAddressingMode]; end;

function OpCode(s: string): TOpCode;
begin
   s := UpperCase(s);
   for Result := Low(OpCodeLookup) to High(OpCodeLookup) do
    if (OpCodeLookup[Result] = s) then
      Exit;
   // special case hard-coded
   if (s = 'CMP') then
    Result := oSEQ
   else
    raise EAssembler.CreateFmt('unknown opcode mnemonic ''%s''',[s]);
end;

function OpModifier(s: string): TOpModifier;
begin
  s := UpperCase(s);
  for Result := Low(OpModifierLookup) to High(OpModifierLookup) do
    if (OpModifierLookup[Result] = s) then
      Exit;
  raise EAssembler.CreateFmt('unknown op-modifier mnemonic ''%s''',[s]);
end;

function AddrMode(ch: char): TAddressingMode;
begin
  for Result := Low(AddressingModeLookup) to High(AddressingModeLookup) do
    if (AddressingModeLookup[Result] = ch) then
      Exit;
  raise EAssembler.CreateFmt('unknown addressing mode mnemonic ''%s''',[ch]);
end;

function Value(s: string): TOperand;
var
  Code: integer;
begin
  Val(s,Result,Code);
  if (Code > 0) then
    raise EAssembler.CreateFmt('invalid value ''%s''',[s]);
end;

end.
