unit ExplainAddressing;

(***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************)

interface

uses
  Mars;

type
  TExplainField = (efINST,efAFIELD,efBFIELD);
  TExplainLine = (elDIRECT,elINDIRECT);
  TExplainIcon = (eiNONE,eiINC,eiDEC);
  TExplainArrow = record
      Src, Dest: record
          Idx: TOperand;
          Field: TExplainField;
        end;
      Icon: TExplainIcon;
      Line: TExplainLine;
    end;
  TExplaination = record
      A, B: TOperand; // the computed value of these fields
      Arrows: array of TExplainArrow;
    end;

procedure Explain(const Mars: TMars; const Idx: TOperand; var Explaination: TExplaination);

implementation

uses
  SysUtils;

procedure Explain(const Mars: TMars; const Idx: TOperand; var Explaination: TExplaination);
var
  Arrow: TExplainArrow;
  IR: TInstruction;
procedure AddArrow;
var
  i: integer;
begin
  with Explaination do begin
    i := Length(Arrows);
    SetLength(Arrows,Succ(i));
    Arrows[i] := Arrow; // copy
  end;
end;
begin
  with Explaination do begin
    SetLength(Arrows,0); // clear arrows
    IR := Mars.Core[Idx];
    // calculate the A arrows
    Arrow.Src.Idx := Idx;
    Arrow.Src.Field := efAFIELD;
    Arrow.Icon := eiNONE;
    Arrow.Line := elDIRECT;
    if (IR.A.AddrMode = aImmediate) then begin
      Arrow.Dest.Idx := 0;
      Arrow.Dest.Field := efINST;
      A := IR.A.Value;
      AddArrow;
    end else if (IR.A.AddrMode = aDirect) then begin
      Arrow.Dest.Idx := Mars.Fold(Idx+IR.A.Value);
      Arrow.Dest.Field := efINST;
      A := Mars.Core[Arrow.Dest.Idx].Index;
      AddArrow;
    end else begin
      // direct part
      case IR.A.AddrMode of
        aPreDecA,
        aPreDecB: Arrow.Icon := eiDEC;
        aPostIncA,
        aPostIncB: Arrow.Icon := eiINC;
      end;
      Arrow.Dest.Idx := Mars.Fold(Idx+IR.A.Value);
      case IR.A.AddrMode of
        aIndirectA,
        aPreDecA,
        aPostIncA:
          begin // is an A mode?
            A := Mars.Core[Arrow.Dest.Idx].A.Value;
            Arrow.Dest.Field := efAFIELD;
          end;
      else
        begin // is therefore a B mode..
          A := Mars.Core[Arrow.Dest.Idx].B.Value;
          Arrow.Dest.Field := efBFIELD;
        end;
      end;
      AddArrow;
      // indirect part?
      case IR.A.AddrMode of
        aPreDecA,
        aPreDecB: Dec(A);
      end;
      Arrow.Src := Arrow.Dest;
      Arrow.Icon := eiNONE;
      Arrow.Line := elINDIRECT;
      Arrow.Dest.Idx := Mars.Fold(Arrow.Src.Idx + A);
      Arrow.Dest.Field := efINST;
      AddArrow;
    end
  end;
end;

end.
