unit FrmCoreUnit;

(***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************)

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls,CoreOverview,CoreDetails,Mars, StdCtrls, Buttons,
  Breakpoint, BreakpointList, BreakpointEditor, WarriorsList;

type
  TFrmCore = class(TForm)
    pnlDetail: TPanel;
    splDetail: TSplitter;
    pnlCore: TPanel;
    pnlBottom: TPanel;
    splStats: TSplitter;
    sclCore: TScrollBox;
    Panel1: TPanel;
    btnRun: TBitBtn;
    btnStep: TBitBtn;
    pnlBreakpoints: TPanel;
    splBreakpoints: TSplitter;
    pnlStats: TPanel;
    lblWarriors: TLabel;
    lblBreakpoints: TPanel;
    btnRemoveBreakpoint: TPanel;
    btnAddBreakpoint: TPanel;
    procedure btnRunClick(Sender: TObject);
    procedure btnStepClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnAddBreakpointClick(Sender: TObject);
  private
    { Private declarations }
    FMars: TMars;
    FBreakpoints: TBreakpointManager;
    FCoreOverview: TCoreOverview;
    FCoreDetails: TCoreDetails;
    FBreakpointList: TBreakpointList;
    FWarriorsList: TWarriorsList;
    procedure SetMars(AMars: TMars);
    procedure SetBreakpoints(ABreakpoints: TBreakpointManager);
    procedure IdleHandler(Sender: TObject; var Done: boolean);
    procedure OnOverviewClick(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
  public
    { Public declarations }
    property Mars: TMars read FMars write SetMars;
    property Breakpoints: TBreakpointManager read FBreakpoints write SetBreakpoints;
  end;

var
  FrmCore: TFrmCore;

implementation

{$R *.dfm}

procedure TFrmCore.SetMars(AMars: TMars);
begin
  FMars := AMars;
  // free any old controls
  if Assigned(FCoreOverview) then
    FCoreOverview.Free;
  if Assigned(FCoreDetails) then begin
    if Assigned(FBreakpointList) then
      FBreakpointList.CoreDetails := nil;
    FCoreDetails.Free;
  end;
  if Assigned(FWarriorsList) then
    FWarriorsList.Free;
  // create overview
  FCoreOverview := TCoreOverview.Create(FMars,Self);
  with FCoreOverview do begin
    Parent := sclCore;
    Align := alClient;
    OnMouseUp := OnOverviewClick;
  end;
  // create warriors list
  FWarriorsList := TWarriorsList.Create(FMars,pnlStats,Self);
  with FWarriorsList do begin
    Align := alClient;
    // todo: add onclick handler?
  end;
  // create details
  FCoreDetails := TCoreDetails.Create(FMars,Self);
  with FCoreDetails do begin
    Parent := pnlDetail;
    Align := alClient;
  end;
  if Assigned(FBreakpointList) then
    FBreakpointList.CoreDetails := FCoreDetails;
end;

procedure TFrmCore.SetBreakpoints(ABreakpoints: TBreakpointManager);
begin
  FBreakpoints := ABreakpoints;
  // free any old controls
  if Assigned(FBreakpointList) then
    FBreakpointList.Free;
  // create new list
  FBreakpointList := TBreakpointList.Create(FBreakpoints,pnlBreakpoints,Self);
  FBreakpointList.Align := alClient;
  if Assigned(FCoreDetails) then
    FBreakpointList.CoreDetails := FCoreDetails;
end;

procedure TFrmCore.OnOverviewClick(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Idx: TOperand;
  Half: integer;
begin
  if (FMars.State = RUN) then // view disabled in this mode?
    Exit;
  with FCoreOverview do begin
    Idx := PointToIndex(X,Y);
    if (Idx < 0) then
      SetSelection(-1,-1) // no selection
    else begin
      FCoreDetails.Focus(Idx);
      Half := FCoreDetails.Rows div 2;
      SetSelection(Idx-Half,Idx+Half);
    end;
  end;
end;

procedure TFrmCore.btnRunClick(Sender: TObject);
begin
  FMars.State := RUN;
  FCoreOverview.SetSelection(-1,-1); // no selection
  FCoreDetails.Focus(FCoreDetails.PC); // to sync it because it uses fast drawing which can become confused ;-)
  FMars.RunStep;
end;

procedure TFrmCore.btnStepClick(Sender: TObject);
begin
  if (FMars.State in [QUICK_RUN,RUN,RUN_STEP]) then begin
    FMars.State := RUN_STEP;
    FMars.RunStep;
  end else if Assigned(FCoreDetails) then
    FCoreDetails.Repaint;
end;


procedure TFrmCore.FormCreate(Sender: TObject);
begin
  FMars := nil;
  FCoreOverview := nil;
  FCoreDetails := nil;
  FBreakpointList := nil;
  Application.OnIdle := IdleHandler;

end;

procedure TFrmCore.IdleHandler(Sender: TObject; var Done: boolean);
begin
  if (FMars.State = RUN) then
    FMars.RunStep;
end;

procedure TFrmCore.btnAddBreakpointClick(Sender: TObject);
begin
  with FrmEditBreakpoint do begin
    Caption := 'Add new Breakpoint';
    ShowModal;
  end;
end;

end.
